"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.paramPathToOpenApiParamObject = exports.parsePathByFramework = exports.buildResponsesPerNamedRoute = exports.buildRouteDefinitions = exports.buildRouteDefinition = void 0;
const core_types_1 = require("./types/core.types");
const wrappedInBrackets = (p) => p[0] === "{" && p[p.length - 1] === "}";
const getParamKey = (p) => p.split("{")[1].split("|")[0];
const wrapParameterByFramework = (p, framework) => {
    if (wrappedInBrackets(p)) {
        if (framework === "express")
            return `:${getParamKey(p)}`;
        else if (framework === "openapi")
            return `{${getParamKey(p)}}`;
    }
    else {
        return p;
    }
};
const buildResponsesPerMethod = (route) => {
    const methods = Object.values(core_types_1.ZemiMethod);
    return methods.map((method) => {
        const hd = route[method];
        if (hd && hd.responses) {
            return { [method]: Object.keys(hd.responses) };
        }
    });
};
const buildChildDefs = (route, parent, name, path) => {
    if (route.routes) {
        const toReturn = Object.assign({}, buildRouteDefinitions(route.routes, { name, path }), parent);
        return toReturn;
    }
    else {
        return parent;
    }
};
const buildDef = (route, prefix) => {
    if (route.name) {
        const name = prefix ? `${prefix.name}-${route.name}` : route.name;
        const dirtyPath = prefix ? `${prefix.path}${route.path}` : route.path;
        const path = parsePathByFramework(dirtyPath, "express");
        const mine = {
            [name]: buildRouteDefinition(path, name),
        };
        return buildChildDefs(route, mine, name, path);
    }
};
const buildChildNamedRoutes = (route, parent, name) => {
    if (route.routes) {
        return Object.assign({}, buildResponsesPerNamedRoute(route.routes, name), parent);
    }
    else {
        return parent;
    }
};
/**
 * Builds a route-definition with the name and path specified.
 * Will build an array of strings with the parameters specified in the path, and
 * a reverse function -- which will substitute params in the path for values
 * passed in via an object -- to the route-definition.
 * @param path {string} - The path for this route-definition.
 * @param name {string} - The name for this route-definition.
 * @return {ZemiRouteDefinition} - A route-definition with the name, path, parameters in the path, and reverse function for said path.
 * @type{(path: string, name:string)=> ZemiRouteDefinition}
 */
function buildRouteDefinition(path, name) {
    const pathArray = path.split("/");
    const parameters = pathArray
        .filter((p) => p.includes(":"))
        .map((p) => p.split(":")[1]);
    const reverse = (paramValues) => {
        const pathParts = pathArray.map((p) => {
            if (p.includes(":")) {
                const key = p.split(":")[1];
                return paramValues[key];
            }
            else
                return p;
        });
        return pathParts.join("/");
    };
    return { name, path, parameters, reverse };
}
exports.buildRouteDefinition = buildRouteDefinition;
/**
 * Recursive. Builds a route-definition for each ZemiRoute in the array.
 * Converts zemi paths (path that has params with their types built in)
 * into valid Express paths. Used to add route-definitions to ZemiRequest
 * object.
 * @param routes {Array<ZemiRoute>} - An array of ZemiRoutes.
 * @param [prefix] {NamedRoute} - The prefix to use for this route, if nested, so that its parent's path is added to its path.
 * @returns {Record<string, ZemiRouteDefinition>} - An object where each key is a route name and the value is that routes' route-definition.
 * @type{(routes: Array<ZemiRoute>, prefix?: NamedRoute)=> Record<string, ZemiRouteDefinition>}
 */
function buildRouteDefinitions(routes, prefix) {
    const namedRoutes = routes.map((route) => buildDef(route, prefix));
    return Object.assign({}, ...namedRoutes);
}
exports.buildRouteDefinitions = buildRouteDefinitions;
/**
 * Recursive. Builds an object with route names as keys and values are objects mapping
 * HTTP methods to array of response codes for those same methods. Used to add
 * allowedResponseCodes to ZemiRequest object.
 * @param routes {Array<ZemiRoute>} - An array of ZemiRoutes.
 * @param [prefix] {NamedRoute} - The prefix to use for this route, if nested, so that its parent's path is added to its path.
 * @returns {Record<string, Record<string, Array<string>>>} - An object where each key is a route name and the value is an object whose keys map to an array of allowed response codes, as strings.
 * @type{(routes: Array<ZemiRoute>, prefix?: NamedRoute)=> Record<string, Record<string, Array<string>>>}
 */
function buildResponsesPerNamedRoute(routes, prefix) {
    const namedRoutes = routes.map((r) => {
        if (r.name) {
            const name = prefix ? `${prefix}-${r.name}` : r.name;
            const responsesPerMethod = buildResponsesPerMethod(r);
            const mine = {
                [name]: Object.assign({}, ...responsesPerMethod),
            };
            return buildChildNamedRoutes(r, mine, name);
        }
    });
    return Object.assign({}, ...namedRoutes);
}
exports.buildResponsesPerNamedRoute = buildResponsesPerNamedRoute;
/**
 * Converts a zemi-style path -- where parameters are specified as
 * `{param|type}` -- to a valid ExpressJS (parameters prefixed by ':') or
 * OpenApi path (parameters wrapped as `{param}`).
 * @param path {string} - A path that has parameters specified as `{param|type}`.
 * @param framework {"express" | "openapi"} - How and for what to parse this param: ExpressJS format or OpenApi format
 * @return {string} - A valid path for the specified framework, that has parameters wrapped accordingly.
 * @type{(path: string)=> string}
 */
function parsePathByFramework(path, framework) {
    const pathBits = path
        .split("/")
        .map((p) => wrapParameterByFramework(p, framework));
    return pathBits.join("/");
}
exports.parsePathByFramework = parsePathByFramework;
/**
 * Converts a zemi-style path -- where parameters are specified as
 * `{param|type}` -- to an OpenApi ParameterObject.
 * Used when generating ParameterObjects, from ZemiRoutes, for OpenApi spec generation.
 * @param path {string} - A path that has parameters specified as `{param|type}`.
 * @return {Array<OpenApiParameterObject>} - An array of OpenApi ParameterObject.
 * @type{(path: string)=> Array<OpenApiParameterObject>}
 */
function paramPathToOpenApiParamObject(path) {
    const paramBits = path
        .split("/")
        .filter((p) => p[0] === "{" && p[p.length - 1] === "}");
    const noBracks = paramBits.map((pb) => pb.substring(1, pb.length - 1));
    const paramsSeparatedList = noBracks.map((p) => p.split("|"));
    return paramsSeparatedList.map(([name, type]) => {
        return {
            name,
            in: "path",
            schema: { type },
            required: true,
        };
    });
}
exports.paramPathToOpenApiParamObject = paramPathToOpenApiParamObject;
//# sourceMappingURL=helpers.js.map